# -*- coding: utf-8 -*-

# -----------------------------------------------------------------------------
# Copyright (c) 2016-2017 Anaconda, Inc.
#
# May be copied and distributed freely only as part of an Anaconda or
# Miniconda installation.
# -----------------------------------------------------------------------------

"""
This folder contains data files bundled with Anaconda Navigator package.

This folder is defined as a python module so that some convenience global variables can be defined.
"""

from __future__ import annotations

__all__ = [
    'CONF_METADATA_PATH', 'DATA_PATH',
    'APP_OVERRIDES_CONF_PATH', 'BASE_APPS_CONF_PATH', 'BUNDLE_METADATA_COMP_PATH', 'LINKS_INFO_PATH',
    'WHATS_NEW_EMPTY_PAGE',
]

import os
import typing

from anaconda_navigator.config import METADATA_PATH


CONF_METADATA_PATH: typing.Final[str] = os.path.join(METADATA_PATH, 'metadata.json')
DATA_PATH: typing.Final[str] = os.path.dirname(os.path.realpath(__file__))

APP_OVERRIDES_CONF_PATH: typing.Final[str] = os.path.join(DATA_PATH, 'app_overrides.yml')
BASE_APPS_CONF_PATH: typing.Final[str] = os.path.join(DATA_PATH, 'base_apps.yml')
BUNDLE_METADATA_COMP_PATH: typing.Final[str] = os.path.join(DATA_PATH, 'metadata.json.bz2')
LINKS_INFO_PATH: typing.Final[str] = os.path.join(DATA_PATH, 'links.json')
WHATS_NEW_EMPTY_PAGE: typing.Final[str] = os.path.join(DATA_PATH, 'whats_new_empty.html')



# -*- coding: utf-8 -*-

# pylint: disable=invalid-name,no-name-in-module

# -----------------------------------------------------------------------------
# Copyright (c) 2016-2017 Anaconda, Inc.
#
# May be copied and distributed freely only as part of an Anaconda or
# Miniconda installation.
# -----------------------------------------------------------------------------

"""Widgets to list videos available to launch on the learning tab."""

from __future__ import absolute_import, division

import os
import sys
from qtpy.QtCore import QPoint, QRect, QSize, QSizeF, Qt, QTimeLine, QUrl, Signal
from qtpy.QtGui import QBrush, QColor, QDesktopServices, QFont, QPainter, QPen, QTextDocument
from qtpy.QtWidgets import QApplication, QHBoxLayout, QListWidget, QVBoxLayout
from anaconda_navigator.utils.styles import SASS_VARIABLES
from anaconda_navigator.utils import attribution
from anaconda_navigator.utils import telemetry
from anaconda_navigator.widgets import ButtonBase, FrameBase, QLabel
from anaconda_navigator.widgets.dialogs import LabelBase
from anaconda_navigator.widgets.lists import ListWidgetBase, ListWidgetItemBase


# --- Widgets used in styling
# -----------------------------------------------------------------------------

class FrameContent(FrameBase):
    """Widget used in CSS styling."""


class LabelContentIcon(QLabel):  # pylint: disable=too-few-public-methods
    """Label used in CSS styling."""


class LabelEmpty(LabelBase):
    """Label used in CSS styling."""


class LabelContentTitle(LabelBase):
    """Label used in CSS styling."""


class ButtonContentText(ButtonBase):
    """Label used in CSS styling."""

    sig_entered = Signal()
    sig_left = Signal()

    def enterEvent(self, event):
        """Override Qt method."""
        super().enterEvent(event)
        self.sig_entered.emit()

    def leaveEvent(self, event):
        """Override Qt method."""
        super().leaveEvent(event)
        self.sig_left.emit()

    def focusInEvent(self, event):
        """Override Qt method."""
        super().focusInEvent(event)
        self.sig_entered.emit()

    def focusOutEvent(self, event):
        """Override Qt method."""
        super().focusOutEvent(event)
        self.sig_left.emit()


class ButtonContentInformation(ButtonBase):
    """Button used in CSS styling."""


class FrameContentHeader(FrameBase):
    """Frame used in CSS styling."""


class FrameContentBody(FrameBase):
    """Label used in CSS styling."""


class FrameContentIcon(FrameBase):
    """Label used in CSS styling."""


class FrameContentHover(FrameBase):  # pylint: disable=too-many-instance-attributes
    """Frame used in css styling with fade in and fade out effect."""

    sig_clicked = Signal()
    sig_entered = Signal()
    sig_left = Signal()

    def __init__(self, *args, **kwargs):
        """Frame used in css styling with fade in and fade out effect."""
        super().__init__(*args, **kwargs)

        self.current_opacity = 0
        self.max_frame = 100
        self.max_opacity = 0.95
        self.button_text = None
        self.label_icon = None
        self.label_text = None
        self.summary = ''

        # Widgets
        self.text_document = QTextDocument(self)
        self.timeline = QTimeLine(500)

        font = QFont()

        if sys.platform == 'darwin':
            font.setPointSize(12)
        elif os.name == 'nt':
            font.setPointSize(8)
        else:
            font.setPointSize(8)

        self.text_document.setDefaultFont(font)
        self.text_document.setMaximumBlockCount(5)
        self.text_document.setDocumentMargin(10)
        # Setup
        self.setWindowFlags(Qt.FramelessWindowHint)
        self.setAutoFillBackground(True)
        self.setWindowFlags(Qt.FramelessWindowHint)
        self.setMinimumSize(self.widget_size())
        self.timeline.setFrameRange(0, self.max_frame)

        # Signals
        self.timeline.frameChanged.connect(self.set_opacity)

    def set_opacity(self, val):
        """Set the opacity via timeline."""
        self.current_opacity = (val * 1.0 / self.max_frame) * self.max_opacity
        self.update()

    def widget_size(self):
        """Return hover frame size."""
        bheight = 40 if self.button_text is None else self.button_text.height()

        width = SASS_VARIABLES.WIDGET_CONTENT_TOTAL_WIDTH
        height = SASS_VARIABLES.WIDGET_CONTENT_TOTAL_HEIGHT
        padding = SASS_VARIABLES.WIDGET_CONTENT_PADDING
        height_margin = 10
        return QSize(width - 2 * padding, height - height_margin - bheight)

    def fade_in(self):
        """Fade in hover card with text."""
        self.raise_()
        self.timeline.stop()
        self.timeline.setDirection(QTimeLine.Forward)
        self.timeline.start()

    def fade_out(self):
        """Fade out hover card with text."""
        self.timeline.stop()
        self.timeline.setDirection(QTimeLine.Backward)
        self.timeline.start()

    def enterEvent(self, event):
        """Override Qt method."""
        super().enterEvent(event)
        self.fade_in()
        self.sig_entered.emit()

    def leaveEvent(self, event):
        """Override Qt method."""
        super().leaveEvent(event)
        self.fade_out()
        self.sig_left.emit()

    def mousePressEvent(self, event):  # pylint: disable=unused-argument
        """Override Qt method."""
        self.sig_clicked.emit()

    def paintEvent(self, event):  # pylint: disable=unused-argument
        """Override Qt method."""
        painter = QPainter(self)
        painter.setOpacity(self.current_opacity)

        max_width = (
            SASS_VARIABLES.WIDGET_CONTENT_TOTAL_WIDTH - 2 * SASS_VARIABLES.WIDGET_CONTENT_PADDING -
            2 * SASS_VARIABLES.WIDGET_CONTENT_MARGIN
        )

        # Hover top
        br = self.rect().bottomRight()
        tl = self.rect().topLeft() + QPoint(1, 1)
        y = br.y() + self.label_text.height() - 2
        br_new = QPoint(max_width - 1, y) - QPoint(1, 1)
        rect_hover = QRect(tl, br_new)  # 2 is the border

        pen = QPen(Qt.NoPen)
        brush = QBrush(Qt.SolidPattern)
        brush.setColor(QColor('#fff'))
        painter.setBrush(brush)
        painter.setPen(pen)
        painter.drawRect(rect_hover)

        font = self.font()
        font.setPointSize(10)
        painter.setFont(font)
        pen = QPen()
        pen.setColor(QColor('black'))
        painter.setPen(pen)
        td = self.text_document
        td.setPageSize(QSizeF(rect_hover.size()))
        td.setHtml(self.summary)
        td.drawContents(painter)

        self.raise_()


def split_text(text, widget, max_width, max_lines=3):
    """Split text in lines according to the widget forn metrics and width."""
    fm = widget.fontMetrics()
    words, lines = [], []

    for word in text.split():
        words.append(word)
        cum_width = fm.width(' '.join(words))
        if cum_width > max_width:
            words.pop()
            lines.append(words)
            words = [word]

    if words:
        lines.append(words)

    if len(lines) > max_lines:
        trimmed_lines = lines[:max_lines]
        trimmed_lines[-1][-1] = '...'  # Replace the last word with ...
    else:
        trimmed_lines = lines

    line_breaks = max_lines - len(trimmed_lines)
    if line_breaks > 0:
        trimmed_lines.append(['\n' * line_breaks])

    text = ''
    for line in trimmed_lines:
        text += ' '.join(line) + '\n'
    return text


# --- Main widgets
# -----------------------------------------------------------------------------
class ListWidgetContent(ListWidgetBase):
    """List Widget holding available videos in the learning tab."""

    sig_view_video = Signal(str, str)

    def __init__(self, *args, **kwargs):
        """List Widget holding available videos in the learning tab."""
        self._main = kwargs.pop('main', None)
        self.utm_medium = kwargs.pop('utm_medium', None)
        super().__init__(*args, **kwargs)
        self.setViewMode(QListWidget.IconMode)

    def ordered_widgets(self):
        """Return a list of the ordered widgets."""
        ordered_widgets = []
        for item in self.items():
            ordered_widgets += item.ordered_widgets()
        return ordered_widgets

    def setup_item(self, item):
        """Override base method."""
        max_width = (
            SASS_VARIABLES.WIDGET_CONTENT_TOTAL_WIDTH - 2 * SASS_VARIABLES.WIDGET_CONTENT_PADDING -
            2 * SASS_VARIABLES.WIDGET_CONTENT_MARGIN
        )
        uri = item.uri
        title = item.title

        item.button_text.clicked.connect(lambda: self.launch(uri, title))
        item.button_text.sig_entered.connect(item.frame_hover.fade_in)
        item.button_text.sig_entered.connect(lambda: self.scroll_to_item(item))
        item.button_text.sig_left.connect(item.frame_hover.fade_out)
        item.frame_hover.sig_clicked.connect(lambda: self.launch(uri, title))
        item.frame_hover.sig_clicked.connect(item.button_text.setFocus)
        item.label_text.setText('\n' + split_text(title, item.label_text, max_width))

    def launch(self, uri: str, title: str) -> None:
        """Emit signal with youtube video identifier string."""
        uri = attribution.POOL.settings.inject_url_parameters(
            uri,
            utm_medium=self.utm_medium or 'tab',
            utm_content=title.lower().replace(' ', '-'),
        )
        qurl = QUrl(uri)
        QDesktopServices.openUrl(qurl)
        telemetry.ANALYTICS.instance.event('redirect', {'url': str(uri)})
        self.sig_view_video.emit(uri, title)


class ListItemContent(ListWidgetItemBase):  # pylint: disable=too-many-instance-attributes
    """Widget to build an item for the content listing."""
    def __init__(  # pylint: disable=too-many-arguments,too-many-locals,too-many-statements
        self,
        title='',
        description='',  # pylint: disable=unused-argument
        uri='',
        authors=None,
        venue='',
        path='',
        year='',
        summary='',
        banner='',
        tags='',
        subtitle='',
        date='',
    ):
        """Widget to build an item for the content listing."""
        super().__init__()

        self.title = title
        self.uri = uri
        self.authors = authors if authors else []
        self.venue = venue
        self.banner = banner
        self.year = year
        self.path = path
        self.tags = tags
        self.subtitle = subtitle
        self.date = date
        self.summary = summary
        self.label = None

        # Widgets
        self.widget = FrameContent()
        self.frame_hover = FrameContentHover(parent=self.widget)
        self.frame_body = FrameContentBody(parent=self.widget)
        self.frame_icon = FrameContentIcon(parent=self.widget)
        self.label_icon = LabelContentIcon(parent=self.widget)
        self.label_icon.setProperty('icon', 'default')
        self.label_text = LabelContentTitle(parent=self.widget)
        self.button_text = ButtonContentText()

        # Widget setup
        self.button_text.setDefault(True)
        self.button_text.setAutoDefault(True)
        self.frame_hover.move(QPoint(5, 5))
        self.frame_hover.label_icon = self.label_icon
        self.frame_hover.label_text = self.label_text
        self.frame_hover.button_text = self.button_text

        valid_tags = {
            'documentation': 'Read',
            'webinar': 'Explore',
            'event': 'Learn More',
            'video': 'View',
            'training': 'Explore',
            'forum': 'Explore',
            'social': 'Engage'
        }
        self.tag = 'notag'
        filter_tags = []
        if len(tags) >= 1:
            filter_tags = [t.lower() for t in tags if t.lower() in valid_tags]
            if filter_tags:
                self.tag = filter_tags[0].lower()

        self.widget.setObjectName(self.tag)
        self.button_text.setObjectName(self.tag)
        self.button_text.setText(valid_tags.get(self.tag, ''))
        self.label_icon.setAlignment(Qt.AlignHCenter)

        # Layout
        if title:
            layout_icon = QVBoxLayout()
            layout_icon_h = QHBoxLayout()
            layout_icon_h.addWidget(self.label_icon)
            layout_icon.addStretch()
            layout_icon.addLayout(layout_icon_h)
            layout_icon.addStretch()
            self.frame_icon.setLayout(layout_icon)

            layout_frame = QVBoxLayout()
            layout_frame.addWidget(self.frame_icon)
            layout_frame.addStretch()
            layout_frame.addWidget(self.label_text)
            layout_frame.addStretch()
            layout_frame.addWidget(self.button_text)
            self.frame_body.setLayout(layout_frame)

            layout = QVBoxLayout()
            layout.addWidget(self.frame_body)
            self.widget.setLayout(layout)
            self.setSizeHint(self.widget_size())
            self.widget.setMinimumSize(self.widget_size())

        if summary:
            date = '<small>' + date + '</small><br>' if date else ''
            sub = '<small>' + subtitle + '</small><br>' if subtitle else ''
            tt = '<p><b>' + title + '</b><br>' + sub + date + summary + '</p>'
            self.frame_hover.summary = tt
        else:
            self.frame_hover.summary = '<p><b>' + title + '</b><br>'

        # Signals
        self.frame_hover.sig_entered.connect(lambda: self.label_text.setProperty('active', True))
        self.frame_hover.sig_left.connect(lambda: self.label_text.setProperty('active', False))
        self.frame_hover.sig_entered.connect(lambda: self.button_text.setProperty('active', True))
        self.frame_hover.sig_left.connect(lambda: self.button_text.setProperty('active', False))

    def ordered_widgets(self):
        """Return a list of the ordered widgets."""
        return [self.button_text]

    def show_information(self):
        """Display additional information of item."""
        if self.label:
            self.label.move(-1000, 0)
            self.label.show()
            app = QApplication.instance()
            geo = app.desktop().screenGeometry(self.button_information)
            w, h = geo.right(), geo.bottom()
            pos = self.button_information.mapToGlobal(QPoint(0, 0))
            x, y = pos.x() + 10, pos.y() + 10
            x = min(x, w - self.label.width())
            y = min(y, h - self.label.height())
            self.label.move(x, y)

    @staticmethod
    def widget_size():
        """Return the size defined in the SASS file."""
        return QSize(SASS_VARIABLES.WIDGET_CONTENT_TOTAL_WIDTH, SASS_VARIABLES.WIDGET_CONTENT_TOTAL_HEIGHT)

    def update_thumbnail(self, pixmap=None):
        """Update thumbnails image."""
        height = SASS_VARIABLES.WIDGET_CONTENT_TOTAL_HEIGHT / 2
        image_width = (
            SASS_VARIABLES.WIDGET_CONTENT_TOTAL_WIDTH - 2 * SASS_VARIABLES.WIDGET_CONTENT_PADDING -
            2 * SASS_VARIABLES.WIDGET_CONTENT_MARGIN
        )
        # image_height = height * 1.666 if 'video' in self.tag else height
        if pixmap and not pixmap.isNull():
            pix_width = pixmap.width()
            pix_height = pixmap.height()
            if pix_width * 1.0 / pix_height < image_width * 1.0 / height:
                max_height = height
                max_width = height * (pix_width / pix_height)
            else:
                max_height = image_width * (pix_height / pix_width)
                max_width = image_width

            self.label_icon.setProperty('icon', 'custom')
            self.label_icon.setScaledContents(True)
            self.label_icon.setMaximumWidth(round(max_width))
            self.label_icon.setMaximumHeight(round(max_height))
            self.label_icon.setPixmap(pixmap)



# -*- coding: utf-8 -*-

# -----------------------------------------------------------------------------
# Copyright (c) Spyder Project Contributors
#
# Licensed under the terms of the MIT License
# (see spyder/__init__.py for details)
# -----------------------------------------------------------------------------

"""Running programs utilities."""

from __future__ import print_function

import os


def is_program_installed(basename):
    """
    Return program absolute path if installed in PATH.

    Otherwise, return None
    """
    for path in os.environ['PATH'].split(os.pathsep):
        abspath = os.path.join(path, basename)
        if os.path.isfile(abspath):
            return abspath
    return None



# -*- coding: utf8 -*-

"""Helper utilities to catch anaconda-client issues."""

from __future__ import annotations

__all__ = ['catch_and_notify']

import collections.abc
import contextlib
import warnings

import binstar_client

from anaconda_navigator.utils import notifications


@contextlib.contextmanager
def catch_and_notify(*, reraise: bool = False) -> collections.abc.Iterator[None]:
    """
    Catch exceptions/warnings and init :class:`~MessageBoxInformation` that will be passed to the notification queue.

    The dialog will not be passed to the notification queue if `once` is set to `True` and dialog is already queued.
    """
    try:
        warns: list[warnings.WarningMessage]
        with warnings.catch_warnings(record=True) as warns:
            warnings.filterwarnings('ignore', category=DeprecationWarning)
            yield

        warning: warnings.WarningMessage
        for warning in warns:
            notifications.NOTIFICATION_QUEUE.push(
                message=str(warning.message),  # pylint: disable=no-member
                caption='Anaconda warning',
                tags=('anaconda', 'warning')
            )

    except binstar_client.BinstarError as error:
        notifications.NOTIFICATION_QUEUE.push(
            message=str(error.message),  # pylint: disable=no-member
            caption='Anaconda error',
            tags=('anaconda', 'warning')
        )
        if reraise:
            raise



# -*- coding: utf-8 -*-

"""Custom exceptions for application detectors."""

from __future__ import annotations

__all__ = ['ValidationError']

import contextlib
import typing


Name = typing.Union[str, int]
Field = typing.Union[Name, typing.Iterable[Name]]


def field_tuple(value: Field) -> typing.Tuple[typing.Union[str, int], ...]:
    """
    Convert field name to uniform format.

    Field names might be of next formats:
    - string - key of dictionary item
    - integer - index of sequence item
    - sequence of any combination of string/integer - multi-level key for multi-level data structures

    Uniform format - is the tuple of any combination of string/integer.
    """
    if isinstance(value, (str, int)):
        return (value,)
    return tuple(value)


class ValidationError(Exception):
    """
    Value didn't pass the validation test.

    :param message: Detailed message about the issue.
    :param field: Path to the field with invalid value.

                  On formats: see :func:`~field_tuple` description.
    :param ignore_stack: Treat `field` as the actual final value.

                         Otherwise - all fields registered with :meth:`~ValidationError.with_field` would be prepended
                         to the provided value.

    Example:

    >>> ValidationError(field=['first', 'second']).field
    ('first', 'second')

    >>> with ValidationError.with_field(['first', 1]):
    ...     ValidationError(field='second').field
    ('first', 1, 'second')
    """

    __stack__: typing.ClassVar[typing.Tuple[typing.Union[str, int], ...]] = ()

    def __init__(
            self,
            message: str = '',
            field: Field = (),
            ignore_stack: bool = False,
    ) -> None:
        """Initialize new :class:`~ValidationError` instance."""
        super().__init__(message)

        actual_field: typing.Tuple[typing.Union[str, int], ...] = field_tuple(field)
        if not ignore_stack:
            actual_field = self.__stack__ + actual_field

        self.__message: typing.Final[str] = message
        self.__field: typing.Final[typing.Tuple[typing.Union[str, int], ...]] = actual_field

    @classmethod
    @contextlib.contextmanager
    def with_field(cls, name: Field) -> typing.Iterator[None]:
        """
        Mark, that field with the `name` is being validated.

        :param name: Name of the field being validated.

                     On formats: see :func:`~field_tuple` description.

        Examples:

        >>> with ValidationError.with_field('external'):
        ...     raise ValidationError(message='message')
        Traceback (most recent call last):
            ...
        anaconda_navigator.api.external_apps.exceptions.ValidationError: external: message

        >>> with ValidationError.with_field('external'):
        ...     raise ValidationError(message='message', field='internal')
        Traceback (most recent call last):
            ...
        anaconda_navigator.api.external_apps.exceptions.ValidationError: external.internal: message
        """
        size: int = len(cls.__stack__)
        cls.__stack__ += field_tuple(name)
        try:
            yield
        finally:
            cls.__stack__ = cls.__stack__[:size]

    @property
    def details(self) -> str:  # noqa: D401
        """Verbose details about detected issue."""
        message: str = self.__message or 'invalid value'

        field: str = '.'.join(map(str, self.__field))
        if field:
            return f'{field}: {message}'

        return message

    @property
    def field(self) -> typing.Tuple[typing.Union[int, str], ...]:  # noqa: D401
        """Normalized value of a field name."""
        return self.__field

    @property
    def message(self) -> str:  # noqa: D401
        """Message about issue."""
        return self.__message

    def __repr__(self) -> str:
        """Retrieve string representation of the instance."""
        return f'{type(self).__name__}: {self.details}'

    def __str__(self) -> str:
        """Retrieve string representation of the instance."""
        return self.details



# -*- coding: utf-8 -*-

"""Additional utility functions to use with application detectors."""

from __future__ import annotations

__all__ = [
    'is_32bit', 'is_64bit', 'is_linux', 'is_not_linux', 'is_osx', 'is_not_osx', 'is_win', 'is_not_win',
    'CliVersionChecker',
]

import re
import typing

from anaconda_navigator import config as navigator_config
from anaconda_navigator.utils.conda import launch as conda_launch_utils

from .. import detectors
from .. import validation


def is_32bit() -> bool:
    """Check if user runs a 32-bit OS."""
    return navigator_config.BITS_32


def is_64bit() -> bool:
    """Check if user runs a 64-bit OS."""
    return navigator_config.BITS_64


def is_linux() -> bool:
    """Check if user runs Linux OS."""
    return navigator_config.LINUX


def is_not_linux() -> bool:
    """Check if user runs anything but Linux OS."""
    return not navigator_config.LINUX


def is_osx() -> bool:
    """Check if user runs OS X."""
    return navigator_config.MAC


def is_not_osx() -> bool:
    """Check if user runs anything but OS X."""
    return not navigator_config.MAC


def is_win() -> bool:
    """Check if user runs Windows OS."""
    return navigator_config.WIN


def is_not_win() -> bool:
    """Check if user runs anything but Windows OS."""
    return not navigator_config.WIN


class CliVersionChecker(detectors.Filter):  # pylint: disable=too-few-public-methods
    """
    Detect version of the application from the terminal output of the application.

    It executes application with a given set of `arguments` and searches for `pattern` in output. The latter one must
    have a single group, value from which will be used as a result version.
    """

    __slots__ = ('__arguments', '__pattern')

    def __init__(
            self,
            pattern: str = r'(\d+\.[\d\.]+)',
            arguments: typing.Union[str, typing.Iterable[str]] = ('--version',),
    ) -> None:
        """Initialize new :class:`~CliVersionChecker` instance."""
        if isinstance(arguments, str):
            arguments = (arguments,)
        elif not isinstance(arguments, tuple):
            arguments = tuple(arguments)

        self.__arguments: typing.Final[typing.Tuple[str, ...]] = arguments
        self.__pattern: typing.Final[typing.Pattern[str]] = re.compile(pattern)

    @classmethod
    def _parse_configuration(cls, *args: typing.Any, **kwargs: typing.Any) -> detectors.Detector:
        """Parse configuration for this particular :class:`~Detector`."""
        validation.OfLength(0)(args, field='args')

        pattern: str = validation.ItemPopper('pattern', default=r'(\d+\.[\d\.]+)').convert(validation.IS_STR)(kwargs)
        arguments: typing.Tuple[str, ...] = validation.ItemPopper('arguments', default=('--version',)).convert(
            validation.sequence_of(str),
        )(kwargs)

        validation.IsEmptyMapping()(kwargs)

        with validation.catch_exception():
            return cls(pattern=pattern, arguments=arguments)

    def __call__(
            self,
            parent: typing.Iterator[detectors.DetectedApplication],
            *,
            context: detectors.DetectorContext,
    ) -> typing.Iterator[detectors.DetectedApplication]:
        """Iterate through detected applications."""
        application: 'detectors.DetectedApplication'
        for application in parent:
            try:
                executable: str = application.executable
            except AttributeError:
                continue

            stdout: str
            try:
                stdout, _, _ = conda_launch_utils.run_process([executable, *self.__arguments])
                match: typing.Optional[typing.Match[str]] = self.__pattern.search(stdout)
                if match:
                    yield application.replace(version=match.group(1))
                    continue
            except (OSError, LookupError, TypeError, ValueError):
                pass

            yield application
